////////////////////////////////////////////////////////////////////////////////
//
// R/C servo pata-pata kit
// Copyright (c) 2007 Noriaki Mitsunaga
//
////////////////////////////////////////////////////////////////////////////////

#include <hidef.h>
#include "derivative.h"

#define FREQ_BUS              8000000  // 8 MHz
#define NUM_BUS_CYCLES        (FREQ_BUS * 100)/1000000
#define DELAY_X_VALUE         (((((NUM_BUS_CYCLES - 6)*10)/4)+5)/10)-1
#define DELAY_TIME            500  // ms 


// ICSTRM location in Flash
unsigned char ICSTRM_FLASH @0xFFAF;

// Flash security (NVOPT Register)
const unsigned char FLASH_SECURITY  @0xFFBF = 0xFE;

// Function declarations
void PeriphInit(void);
void Delay_100us (unsigned char hundreds_us);


////////////////////////////////////////////////////////////////////////////////
// PeriphInit
// ----------------------------------------------------------------------------
// Initializes various registers and peripherals
////////////////////////////////////////////////////////////////////////////////
void PeriphInit(void)
{  
  //Disables COP
  SOPT1 &= 0x7F;
  
  //Initialize ADC
  APCTL1 = 0x0a; // AD1 & AD3 enabled
  ADCCFG = 0x70; // ADLPC=0(0), ADIV=8(11), ADLSMP = 1(1)
                 // MODE=00(00), ADICLK=00
  ADCSC2 = 0;           
  // Configures PTA0 as output LED, PAT2 as output for R/C servo
  // PTA1(AD1), PTA3(AD3): output
  PTAD = 0x00;
  PTADD = 0x0f;
  
  // Configures ICS clock. Selects FLL Engaged Internal Mode (Divides ICSOUT by 1)
  // fBUS about 8 MHz
  ICSTRM = ICSTRM_FLASH;
  ICSC2 = 0x00;
  ICSC1 = 0x06;
  //Wait until output FLL is selected (CLKST=0)
  while (ICSSC & 0x04)
    ;
}

////////////////////////////////////////////////////////////////////////////////
// Delay_100us
// ----------------------------------------------------------------------------
// This function performs a specified delay in multiples of 100 us
////////////////////////////////////////////////////////////////////////////////
void Delay_100us (unsigned char hundreds_us)
{
  asm 
    {
      lda hundreds_us
      beq exit
  delay_us:                    
      ldx #DELAY_X_VALUE          ; [2] 
  loop:
      dbnzx loop                  ; [4] 
      dbnza delay_us              ; [4]
  exit:
    }
}

////////////////////////////////////////////////////////////////////////////////
// Delay to make pulse for R/C servo
// ----------------------------------------------------------------------------
// This function performs a specified delay in multiples of 5.5 us
////////////////////////////////////////////////////////////////////////////////
void Delay_servo (unsigned char pos)
{         // 11x4 / (8000*1000) = 5.5[us]
  asm 
    {
      ldx pos
      beq exit
 loop:
      nop
      nop
      nop
      nop          ; [4]
      nop
      nop
      nop
      nop          ; [4]
      nop
      nop
      nop
      nop          ; [4]
      nop
      nop
      nop
      nop          ; [4]
      nop
      nop
      nop
      nop          ; [4]
      nop
      nop
      nop
      nop          ; [4]
      nop
      nop
      nop
      nop          ; [4]
      nop
      nop
      nop
      nop          ; [4]
      nop
      nop
      nop
      nop          ; [4]
      nop
      nop
      nop
      nop          ; [4]
      dbnzx loop   ; [4] 
   exit:
    }
}


////////////////////////////////////////////////////////////////////////////////
// main function
////////////////////////////////////////////////////////////////////////////////
void
main(void) 
{
  unsigned char vr, j = 0;

  EnableInterrupts;
  PeriphInit();
  
  for(;;) // Forever
    {     // run in 50Hz   
      ADCSC1 = 0x1;  // ADC conversion start
      while ((ADCSC1 & 0x80) == 0) // Wait for conversion to complete
           	; 
      vr = ADCRL;   

	    // Make a pluse (0.7ms - 2.1ms)
	    PTAD |= 0x4;       // set H
      
	    Delay_100us(7);    // 0.7ms delay
	    Delay_servo(vr);  // 0-1.4ms delay (0x5.5us - 255x5.5us)

	    PTAD &= 0xfb;      // set L

	    // Flash LED
	    j ++;
	    vr = vr/8;
	    if (j > vr) {
	      // Once for vr/8*20ms
	      // Toggles LED
	      PTAD ^= 0x01;
	      j = 0;
      }
      Delay_100us(200); // 20ms delay
    }
}
